// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.cometTennis.message;

import ch.niceideas.cometTennis.TennisManager;
import ch.niceideas.common.http.comet.CometBusinessException;
import ch.niceideas.common.http.comet.CometServerException;
import ch.niceideas.common.http.comet.message.MessageHandler;
import ch.niceideas.common.service.Registry;
import ch.niceideas.common.service.WebRegistry;
import ch.niceideas.common.utils.CommonBusinessException;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import javax.servlet.http.HttpServletRequest;

/**
 * Traps Messages related to the tennis game and forwards them to the 
 * TennisManager
 */
public class TennisMessageHandler implements MessageHandler {

    private static final Logger logger = Logger.getLogger(TennisMessageHandler.class);
    
    /**
     * {@inheritDoc}
     */
    @Override
    public boolean accept(HttpServletRequest request, String streamId, String clientId, String message)
            throws CometServerException, CometBusinessException {
        
        if (   message.startsWith("INIT") 
            || message.startsWith("JOIN")
            || message.startsWith("START_UP")
            || message.startsWith("START_DOWN")
            || message.startsWith("STOP_UP")
            || message.startsWith("STOP_DOWN")) {
            
            if (StringUtils.isBlank(clientId)) {
                logger.warn("No clientId provided. This is unsupported.");
                throw new CometServerException("No clientId provided. This is unsupported.");
            }
            
            try {
                Registry registry = new WebRegistry(request.getSession());
                TennisManager manager = (TennisManager) registry.getService (TennisManager.class.getCanonicalName());
                
                manager.message (clientId, message);
                
            } catch (CometBusinessException e) {
            	logger.warn(e.getMessage(), e);
            	throw new CometBusinessException(e.getMessage(), e);
            	
            } catch (CommonBusinessException e) {
                logger.error (e, e);
                throw new CometServerException(e.getMessage(), e);
            }            
            
            return true;
        }

        return false;
    }

}
