// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.cometTennis.model;

import ch.niceideas.cometTennis.model.TennisGame.GameMove;
import ch.niceideas.common.http.comet.CometServerException;

/**
 * Represents the tennis ball. 
 */
public class TennisBall { 
    
    private static final int BOARD_WIDTH = TennisGame.BOARD_WIDTH;
    private static final int BOARD_HEIGHT = TennisGame.BOARD_HEIGHT;
    
    private static final int BALL_WIDTH = TennisGame.BALL_WIDTH;
    
    private static final int BALL_SPEED_X = TennisGame.BALL_SPEED_X;
    private static final int INIT_BALL_SPEED_Y = TennisGame.INIT_BALL_SPEED_Y;
    
    private static final int PAD_HEIGHT = TennisGame.PAD_HEIGHT;
    private static final int PAD_WIDTH_OFFSET = TennisGame.PAD_WIDTH_OFFSET;
    
    private int posX = (BOARD_WIDTH / 2) - (BALL_WIDTH / 2);
    private int posY = (BOARD_HEIGHT / 2) - (BALL_WIDTH / 2);
    
    private GameMove move = GameMove.LEFT;
    
    private int ballSpeedY = INIT_BALL_SPEED_Y;
    
    public TennisBall() {
    }       
    
    public void period() throws CometServerException {

        if (move.equals(GameMove.LEFT)) {
            
            posX = posX - BALL_SPEED_X;
            posY = posY + ballSpeedY;
            
            if (posY >= BOARD_HEIGHT - BALL_WIDTH) {
                posY = BOARD_HEIGHT - BALL_WIDTH;
                ballSpeedY = -ballSpeedY;
            } else if (posY <= 0) {
                posY = 0;
                ballSpeedY = -ballSpeedY;
            }
            
        } else if (move.equals(GameMove.RIGHT)) {
            
            posX = posX + BALL_SPEED_X;
            posY = posY + ballSpeedY;
            
            if (posY >= BOARD_HEIGHT - BALL_WIDTH) {
                posY = BOARD_HEIGHT - BALL_WIDTH;
                ballSpeedY = -ballSpeedY;
            } else if (posY <= 0) {
                posY = 0;
                ballSpeedY = -ballSpeedY;
            }           
        }             
    }
    
    public GameMove detectWallCollisions() {
        // Detect collisions with left and right walls
        if (posX <= 0) { 
            posX  = (BOARD_WIDTH / 2) - (BALL_WIDTH / 2);
            posY = (BOARD_HEIGHT / 2) - (BALL_WIDTH / 2);
            move = GameMove.LEFT;
            ballSpeedY = INIT_BALL_SPEED_Y;
            return GameMove.RIGHT;
        }
        
        if (posX >= BOARD_WIDTH - BALL_WIDTH) {
            posX = (BOARD_WIDTH / 2) - (BALL_WIDTH / 2);
            posY = (BOARD_HEIGHT / 2) - (BALL_WIDTH / 2);
            move = GameMove.RIGHT;
            ballSpeedY = INIT_BALL_SPEED_Y;
            return GameMove.LEFT;
        }
        
        return null;
    }
    
    public void detectCollisionRightPlayer(TennisPlayer player) {
        if (posY >= player.getPos() - (BALL_WIDTH / 2) && posY <= player.getPos() + PAD_HEIGHT + (BALL_WIDTH / 2)) {
            if (posX >= (BOARD_WIDTH - PAD_WIDTH_OFFSET) - BALL_WIDTH) {                    
                // collision with first player !
                move = GameMove.LEFT;
                
                if (player.isMoveUp()) {
                    ballSpeedY--;
                } else if (player.isMoveDown()) {
                    ballSpeedY++;
                }
            }            
        }
    }

    public void detectCollisionLeftPlayer(TennisPlayer player) {
        if (posY >= player.getPos() - (BALL_WIDTH / 2) && posY <= player.getPos() + PAD_HEIGHT + (BALL_WIDTH / 2)) {
            if (posX <= PAD_WIDTH_OFFSET) {                    
                // collision with first player !
                move = GameMove.RIGHT;
                
                if (player.isMoveUp()) {
                    ballSpeedY--;
                } else if (player.isMoveDown()) {
                    ballSpeedY++;
                }
            }            
        }
    }

    public int getPosX() {
        return posX;
    }

    public void setPosX(int ballPosX) {
        this.posX = ballPosX;
    }

    public int getPosY() {
        return posY;
    }

    public void setPosY(int ballPosY) {
        this.posY = ballPosY;
    }

    public GameMove getMove() {
        return move;
    }

    public void setMove(GameMove ballMove) {
        this.move = ballMove;
    }
   
}
