// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.cometTennis.model;

import ch.niceideas.cometTennis.model.TennisGame.PlayerMove;
import ch.niceideas.common.http.comet.CometServerException;

/**
 * Represents one tennis player
 */
public class TennisPlayer { 
    
    private static final int PAD_STEP = TennisGame.PAD_STEP;
    
    private static final int PAD_HEIGHT = TennisGame.PAD_HEIGHT;
    private static final int BOARD_HEIGHT = TennisGame.BOARD_HEIGHT;
    
    
    private final String id;    
    private PlayerMove move = PlayerMove.NONE;    
    private int pos = 200;
    
    public TennisPlayer(String playerId) {
        this.id = playerId;
    }
    
    boolean matchesId (String clientId) {
        return clientId.matches(this.id);
    }
    
    public void startUp() {
        move = PlayerMove.UP;
    }
    
    public void startDown() {
        move = PlayerMove.DOWN;
    }

    public void stopUp() {
        move = PlayerMove.NONE;
    }
    
    public void stopDown() {
        move = PlayerMove.NONE;
    }   
    
    public boolean isMoveUp() {
        return move == PlayerMove.UP;
    }
    
    public boolean isMoveDown() {
        return move == PlayerMove.DOWN;
    }
    
    public String getId() {
        return id;
    }

    public int getPos() {
        return pos;
    }    
    
    public void period() throws CometServerException {
        // move first player
        if (move.equals(PlayerMove.UP)) {
            pos -= PAD_STEP;
        } else if (move.equals(PlayerMove.DOWN)) {
            pos += PAD_STEP;
        }
        if (pos <= 0) {
            pos = 0;
            move = PlayerMove.NONE;
        }            
        if (pos >= (BOARD_HEIGHT - PAD_HEIGHT)) {
            pos = (BOARD_HEIGHT - PAD_HEIGHT);
            move = PlayerMove.NONE;
        }                
    }
   
}
