// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.http.comet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

public class CometHandle {
    
    private static final Logger logger = Logger.getLogger(CometHandle.class);
    
    private final String streamId;
    private final String clientId;
    private HttpServletResponse response;
    private final PrintWriter writer;
    private final Object monitor;
    
    public CometHandle(String streamId, String clientId, PrintWriter writer, HttpServletResponse response) {
        super();
        this.streamId = streamId;
        this.clientId = clientId;
        this.writer = writer;
        this.response = response;
        this.monitor = new Object();
    }
    
    public String getStreamId() {
        return streamId;
    }
    
    public String getClientId() {
        return clientId;
    }
    
    void waitForCompletion() {
        synchronized (monitor) {
            try {
                logger.info("HTTP Thread " + Thread.currentThread().getName() + " waiting for completion on handle " + this);                
                monitor.wait();
            } catch (InterruptedException e) {
                logger.error (e, e);
            }
        }
    }
    
    void terminate() {
        this.response = null;
        synchronized (monitor) {
            monitor.notify();
        }
    }
    
    public void sendMessage(String message) throws CometServerException {

        if (message.trim().equals("KILL")) {
            sendCommand("COMET_STOP:streamId=" + this.getStreamId() + ":-");
            this.terminate();
            
        } else if (message.trim().equals("RECONNECT")) {
            
            sendCommand("CLIENT_RECONNECT:streamId=" + this.getStreamId() + ":-");
            this.terminate();
            
        } else {
        
            sendCommand("MESSAGE:streamId=" + this.getStreamId() + ":" + message);
        }
    }

    private void sendCommand(String command) throws CometStreamException {
        synchronized (monitor) {
            if (this.response == null) {
                throw new CometCloseException ("Response is closed. Handle is discarded.");
            }
            writer.print ("<script type='text/javascript'>\n");
            writer.print ("processMessage('" + command +  "');\n");
            writer.print ("</script>\n");
            writer.print ("&nbsp;<br />\n"); 
            writer.flush();        
            try {
                response.flushBuffer();
            } catch (IOException e) {
                logger.error (e, e);
                throw new CometCloseException (e.getMessage(), e);
            }
            
            if (writer.checkError()) {
                throw new CometCloseException ("The underlying stream has encountered an error");
            }
        }
    }
    
    /**
     * Debugging ...
     */
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append ("ComentHandler[");
        sb.append ("streamId=");
        sb.append(streamId);
        sb.append ("clientId=");
        sb.append(clientId);
        sb.append ("]");
        return sb.toString();
    }
    

}
