// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.http.comet;

import ch.niceideas.common.service.Registry;
import ch.niceideas.common.service.WebRegistry;
import ch.niceideas.common.utils.CommonBusinessException;
import ch.niceideas.common.utils.ResourceUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.log4j.lf5.util.StreamUtils;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

public class CometStreamServlet extends HttpServlet {
        
    private static final Logger logger = Logger.getLogger(CometStreamServlet.class);
    
    private static final long serialVersionUID = -4717836700342947212L;

    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost (req, resp);
    }
    
    /**
     * !!! CAUTION !!!
     * <p />
     * 
     * Every connected browser here will keep the thread he gets there forever (... until deconnection)
     * <p />
     * 
     * The Tomcat server needs to have a <b>VERY LARGE</b> pool of threads !
     * 
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        
        try {
            Registry registry = new WebRegistry(req.getSession());
            CometManager manager = (CometManager) registry.getService(CometManager.class.getCanonicalName());
            
            String streamId = req.getParameter("streamId");
            if (StringUtils.isBlank(streamId)) {
                throw new CommonBusinessException ("Expected 'streamId' as servlet argument.");
            }
            
            String clientId = req.getParameter("clientId");
            if (StringUtils.isBlank(clientId)) {
                throw new CommonBusinessException ("Expected 'clientId' as servlet argument.");
            }
            
            logger.info("");
            logger.info("--------------------------------------------------------------------------------");
            logger.info("Got incoming stream from clientId=" + clientId + " for streamId=" + streamId + " with thread=" + Thread.currentThread().getName() + " / session=" + req.getSession().getId());
        
            resp.setHeader("Cache-Control", "no-cache, must-revalidate");        
            resp.setHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT"); 
            resp.setContentType("text/html");
            resp.setCharacterEncoding("UTF-8");
            
            PrintWriter writer = resp.getWriter(); 
            writer.print ("<html>\n");
            writer.print ("<head>\n");        
            writer.print ("</head>\n");
            writer.print ("<body>\n");
            
            // fill with whitespace since first 2k are buffered
            for (int i = 0; i < 2000; i++) {
                writer.print (" ");
            }
            
            writer.print ("<script type='text/javascript'>\n");
            writer.print ("\n");
            writer.print ("    var streamId = '" + streamId + "';\n");
            writer.print ("\n");
            
            InputStream is = ResourceUtils.getResourceAsStream("comet_iframe_init_stream.js");
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            StreamUtils.copy(is, baos);
            is.close();
            writer.write(baos.toString("UTF-8"));           
    
            writer.print ("</script>\n");
            
            writer.flush();        
            resp.flushBuffer();
            
            CometHandle handle = new CometHandle(streamId, clientId, writer, resp);
            manager.registerHandle(handle);
            
            // And here we wait
            handle.waitForCompletion();
            logger.info(" - stream dropped from clientId=" + clientId + " for streamId=" + streamId + " with thread=" + Thread.currentThread().getName() + " / session=" + req.getSession().getId());
            
            writer.print ("</body>\n");
            writer.print ("</html>\n");
        
        } catch (Exception e) {
            logger.error ("thread=" + Thread.currentThread().getName() + " / session=" + req.getSession().getId() + " - ERROR : " + e.getMessage(), e);
            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
        }
    }

}
