// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.http.comet.message;

import javax.servlet.http.HttpServletRequest;

import ch.niceideas.common.http.comet.CometBusinessException;
import ch.niceideas.common.http.comet.CometServerException;

/**
 * A message handler registered on the CometMessageServlet.
 * <p />
 * 
 * When a new message arrives, the CometMessageServlet iterates through
 * MessageHandler instances and calls <code>accept</code> on each of them.
 * <br />
 * It stops when a first one answers <code>true</code>.   
 */
public interface MessageHandler {
    
    /**
     * Try to handle the message. The MessageHandler answers <code>false</code> 
     * if it cannot handle this message and <code>true</code> if it has been 
     * able to successfully handle it and the message should not be passed 
     * further.
     * 
     * @param request reference on the HTTP request
     * @param streamId identifier of the stream
     * @param clientId identifier of the connected client (UI(
     * @param message the message to be handled
     * @return true of this specific MessageHandler has been able to handle the 
     *         given message
     * @throws CometServerException if any technical error occurs
     * @throws CometServerException if any business error occurs
     */
    boolean accept (HttpServletRequest request, String streamId, String clientId, String message) 
    		throws CometServerException, CometBusinessException;

}
