// ============================================================================
//
// Copyright (c) 2010 - 2011, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================

package ch.niceideas.common.service;

/**
 * A Registry is a service manager. It stores services following a specific scope passed to the
 * <code>storeService</code> method.
 * <br />
 * Stored services can be retrieved regardless of the scope. They are searched in the smallest scope first and then in
 * larger scopes..
 * <p />
 * 
 * It is not necessary to know the scope in which the service has been stored when trying to retrieve a service.
 * The desired service is first looked up in the local scope, then the user scope, then the context scope and
 * finally the application scope.
 */
public interface Registry {

    enum Scope {
        APPLICATION,
        CONTEXT,
        USER,
        LOCAL
    };

    /**
     * Get a service identified by its key
     * 
     * @param key a key identifiying the desired service
     * @return the corresponding service if any
     */
    Object getService(Object key);

    /**
     * Store a service identified by a key in a specific scope
     * 
     * @param key a key identifiying the service
     * @param scope the scope in which to store the service
     * @param service the service to be stored
     */
    void storeService(Object key, Scope scope, Object service);
}
