// ============================================================================
//
// Copyright (c) 2010 - 2011, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.service;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

/**
 * Just an utility class to store some information related to a service to be stored in a Registry.
 */
public class ServiceWrapper {

    private Class<?> serviceClass;
    private final String serviceName;

    private final Collection<Field> fieldsToInit = new ArrayList<Field>();

    private final Collection<Method> methodsToEnhance = new ArrayList<Method>();

    /**
     * Build a wrapper for a specific service identified by its class
     *
     * @param serviceClass the class of the service to be wrapped.
     */
    public ServiceWrapper(Class<?> serviceClass) {
        this.serviceClass = serviceClass;
        this.serviceName = serviceClass.getCanonicalName();
    }

    public Class<?> getServiceClass() {
        return serviceClass;
    }

    public void overrideClass(Class<?> serviceClass) {
        this.serviceClass = serviceClass;
    }

    public void addFieldToInit (Field field) {
        fieldsToInit.add (field);
    }

    public void addMethodtoEnhance(Method method) {
        methodsToEnhance.add (method);
    }

    public Collection<Field> getFieldsToInit() {
        return Collections.unmodifiableCollection(fieldsToInit);
    }

    public Collection<Method> getMethodsToEnhance() {
        return Collections.unmodifiableCollection(methodsToEnhance);
    }

    public String getServiceName() {
        return this.serviceName;
    }

}
