// ============================================================================
//
// Copyright (c) 2010 - 2011, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================

package ch.niceideas.common.service;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * A static registry stores services in a static map.
 * <br />
 * Only the "APPLICATION" Scope is supported. Attemtping to store a service in another scope results in an exception.
 */
public class StaticRegistry implements Registry {

    private final static Map<String, Object> applicationServices = new HashMap<String, Object>();
    
    public StaticRegistry() {        
    }
    
    /**
     * {@inheritDoc}
     */
    public Object getService(Object key) {
        return applicationServices.get(key.toString());
    }

    Set<String> getServiceKeys() {
        return applicationServices.keySet();
    }

    /**
     * {@inheritDoc}
     */
    public void storeService(Object key, Scope scope, Object service) {
        if (scope.equals(Scope.APPLICATION)) {
            applicationServices.put(key.toString(), service);

        } else if (scope.equals(Scope.CONTEXT)) {
            throw new UnsupportedOperationException("Unsupported Scope : " + scope);

        } else if (scope.equals(Scope.LOCAL)) {
            throw new UnsupportedOperationException("Unsupported Scope : " + scope);
            
        } else {
            throw new UnsupportedOperationException("Unsupported Scope : " + scope);
        }
    }

}
