// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.utils;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * FIXME Document me
 * <p />
 * 
 * <b>
 * The design of this class as well as in general the ideas expressed here and the original implementation of these
 * ideas are the work of Mr. Thomas Beck. <br />
 * I would like to take this opportunity to thank him for the privilege and the great pleasure it has been to work
 * during those years with the very best software engineer I have ever met in my career.
 * </b>
 */
public class CommonBusinessException extends Exception {

    private static final long serialVersionUID = -8017767618562184690L;

    private List<Throwable> underlyingExceptions = null;

    /**
     * FIXME Document me
     */
    public CommonBusinessException() {}

    /**
     * FIXME Document me
     */
    public CommonBusinessException(String message) {
        super(message);
    }

    /**
     * FIXME Document me
     */
    public CommonBusinessException(String message, Throwable under) {
        super(message);
        this.addUnderlyingException(under);
    }

    /**
     * FIXME Document me
     */
    public CommonBusinessException(Throwable under) {
        super();
        this.addUnderlyingException(under);
    }

    /**
     * FIXME Document me
     */
    public void addUnderlyingException(Throwable under) {
        if (under != null) {
            if (underlyingExceptions == null) {
                underlyingExceptions = new ArrayList<Throwable>(1);
            }
            underlyingExceptions.add(under);
        }
    }

    /**
     * FIXME Document me
     */
    public Throwable[] getUnderlyingExceptions() {
        if (underlyingExceptions == null) {
            return new Throwable[0];
        }
        Throwable[] array = new Throwable[underlyingExceptions.size()];
        return underlyingExceptions.toArray(array);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public synchronized Throwable initCause(Throwable cause) {
        return super.initCause(cause);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Throwable getCause() {
        if (underlyingExceptions == null) {
            return null;
        }
        return underlyingExceptions.get(0);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void printStackTrace() {
        System.err.println(getCompleteMessage());
        super.printStackTrace();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void printStackTrace(PrintStream s) {
        s.println(getCompleteMessage());
        super.printStackTrace(s);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void printStackTrace(PrintWriter s) {
        s.println(getCompleteMessage());
        super.printStackTrace(s);
    }

    /**
     * FIXME Document me
     */
    public String getCompleteMessage() {
        StringBuilder messageBuilder = new StringBuilder();
        String message = this.getMessage();
        if (message == null) {
            message = this.getClass().getName();
        }
        messageBuilder.append(message);
        addUnderlyingExceptions(messageBuilder, this, 1);
        return messageBuilder.toString();
    }

    private void addUnderlyingExceptions (StringBuilder messageBuilder, Throwable parent, int level) {

        if (parent.getCause() == null) {
            return;
        }

        if (parent instanceof CommonBusinessException) {
            Throwable[] myUnderlyingExceptions = ((CommonBusinessException) parent).getUnderlyingExceptions();
            for (Throwable under : myUnderlyingExceptions) {
                appendMessage(messageBuilder, level, under);

                // recursive call on each cause
                addUnderlyingExceptions(messageBuilder, under, level + 1);

            }
        } else {

            appendMessage(messageBuilder, level, parent.getCause());

            addUnderlyingExceptions(messageBuilder, parent.getCause(), level + 1);
        }
    }

    private void appendMessage(StringBuilder messageBuilder, int level, Throwable under) {
        String message = under.getMessage();
        if (message == null) {
            message = under.getClass().getName();
        }
        messageBuilder.append("\n");
        for (int l = 0; l < level; l++) {
            messageBuilder.append("  ");
        }
        messageBuilder.append(message);
    }

    /**
     * Extract the stack trace from an exception and put it in a string
     *
     * @param e the concerned exception
     * @return a string representing the stack trace.
     */
    public String stackToString() {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        printStackTrace(pw);
        return sw.toString();
    }
    
    /**
     * Get the Stack-Trace as a string
     * 
     * @return the Stack-Trace
     * @throws IOException
     */
    public String getStackTraceString() {
        return stackToString();
    }

    /**
     * FIXME Document me
     */    
    @SuppressWarnings("unchecked")
    public <T extends CommonBusinessException> void throwIfAny() throws T {
        if (underlyingExceptions != null && underlyingExceptions.size() > 0) {
            throw (T) this;
        }
    }

    /**
     * FIXME Document me
     */
    public void resetUnderlyingExceptions() {
        if (underlyingExceptions != null && underlyingExceptions.size() > 0) {
            underlyingExceptions.clear();
        }
    }
}
