// ============================================================================
//
// Copyright (c) 2010 - 2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


//
// initialization
// ----------------------------------------------------------------------------
// 

var SERVER_COMET_STREAM_ROOT = "serverComet.stream";
var SERVER_COMET_MESSAGE_ROOT = "serverComet.message";

var EXTRACT_COMET_MESSAGE_REXP = "(.*):streamId=([a-zA-Z0-9_\-]*):(.*)";

var MAX_STREAM_DURATION_BEFORE_RECONNECT = 30;

var myClientId = null;
var autoReconnect = {};
var cometInitialized = false;
var reconnectTimeout = {};

//
// functions
// ----------------------------------------------------------------------------
// 

/*
 * Utility functions
 */

isFunction = function (object) {
    return !!(object && object.constructor && object.call && object.apply);
};

extractParams = function (order) {
    
	var paramsRexp = new RegExp(EXTRACT_COMET_MESSAGE_REXP, 'gi')
    var matches =  paramsRexp.exec(order);
    
    var retParams = {
    	order : matches[1],
    	streamId: matches[2],
    	data: matches[3]
    };
    
    return retParams;
};

generateUUID = function (){
    var d = new Date().getTime();
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        var r = (d + Math.random()*16)%16 | 0;
        d = Math.floor(d/16);
        return (c=='x' ? r : (r&0x7|0x8)).toString(16);
    });
    return uuid;
};

stopEvent = function (event) {
    // CANCEL THE EVENT. Just try them all :
    if (event.preventDefault) event.preventDefault();
    if (event.stop) event.stop();               
    if (event.stopPropagation) event.stopPropagation();
    //if (event.preventBubble) event.preventBubble(); // deprecated
    event.returnValue = false;
    event.cancelBubble = true;
    event.done = true;    
};

getXMLHttpRequest = function () {
    var xhr = null;
     
    if (window.XMLHttpRequest || window.ActiveXObject) {
        if (window.ActiveXObject) {
            try {
                xhr = new ActiveXObject("Msxml2.XMLHTTP");
            } catch(e) {
                xhr = new ActiveXObject("Microsoft.XMLHTTP");
            }
        } else {
            xhr = new XMLHttpRequest();
        }
    } else {
        alert("Your navigator doesn't support XMLHTTPRequest...");
        return null;
    }
     
    return xhr;
};

destroyIframe = function(streamId) {
	console.log("destroyIframe", streamId);
	
	clearTimeout (reconnectTimeout[streamId]);
	
    var iFrame = document.getElementById("serverIFrame_" + streamId);
    iFrame.src = "about:blank";
    iFrame.parentNode.removeChild(iFrame);
    delete iFrame;    
};

/**
 * Messaging system : send messages to the server (separate stream from comet)
 */
sendControlMessage = function(streamId, message) {
	console.log("sendControlMessage", streamId, message);
	
	var xhr = getXMLHttpRequest();
	
    xhr.onreadystatechange = function() {
        //document.ajax.dyn.value = "Wait server...";
        if (xhr.readyState == 4) {
            if (xhr.status == 200) {
                // OK
                //alert ("Received:" + xhr.responseText);
            } else if (xhr.status == 566){
            	alert (xhr.responseText);
            	
            } else {
                alert ("Error: returned status code " + xhr.status + " with " + xhr.statusText);
            }
        }
    }; 

    xhr.open("GET", SERVER_COMET_MESSAGE_ROOT + "?streamId=" + streamId + "&clientId=" + myClientId + "&message=" + message, true);                
    xhr.send(null);     
};

/**
 * Messaging system : receive messages from the backend (comet stream)
 */
onServerMessage = function(e) {
    
	if (!e || !e.data) { 
		alert ("Empty message received from backend. This is an exception.");
	}
	
    var params = extractParams (e.data);
    
    if (params.order == "COMET_RECONNECT") {
    	
    	console.log("onServerMessage - COMET_RECONNECT", params); 
    	
    	if (autoReconnect[params.streamId]) {
            setTimeout (function() {
                initServerConnection (params.streamId, true);
            }, 1);            
        }                  
        destroyIframe(params.streamId);
        
    } else if (params.order == "CLIENT_RECONNECT") {
    	
    	console.log("onServerMessage - CLIENT_RECONNECT", params);
    	
    	setTimeout (function() {
            initServerConnection (params.streamId, true);
        }, 1); 
        destroyIframe(params.streamId);
        
    } else if (params.order == "COMET_STOP") {
    	
    	console.log("onServerMessage - COMET_STOP", params);
    	
    	destroyIframe(params.streamId);
    }
    
    // Other cases
    else {                
       
    	// Other cases are expected to be Application messages and are sent to application
    	if (handleServerMessage && isFunction(handleServerMessage)) {
    		handleServerMessage (params.data);
    	} else {
    		alert ("No business message handler is defined. Was expecting a function 'handleServeressage' defined in main frame.");
    	}
        
    }
};

/**
 * Initialize base requirements for comet
 */
initComet = function() {
	console.log("initComet");
	
    if (!cometInitialized) {
        
        // IE
        if (!window.addEventListener) {
            window.addEventListener = function (type, listener, capt) {
                attachEvent("on"+type, function(){listener(event)});
            }
        }

        addEventListener("message", onServerMessage);        
        
        myClientId = generateUUID();
        
        cometInitialized = true;
    }
};

/**
 * This is the trigger of the periodical reconnection of the iframe
 */
initReconnectTimeout = function(streamId) {
	console.log("initReconnectTimeout");
	
	reconnectTimeout[streamId] = setTimeout (function() {	 
		console.log("TIMEOUT - reconnectTimeout", streamId);
		sendControlMessage(streamId, "RECONNECT");	    	
	}, MAX_STREAM_DURATION_BEFORE_RECONNECT * 1000);
};

/**
 * This initializes the whole comet shebang
 */
initServerConnection = function(streamId, doReconnect) {
	console.log("initServerConnection", streamId, doReconnect);
	
    autoReconnect[streamId] = doReconnect;        
    
    if (!cometInitialized) {
    	initComet();
    }
    
    var ifFrame = document.createElement("iframe");
    ifFrame.src = SERVER_COMET_STREAM_ROOT + "?streamId=" + streamId + "&clientId=" + myClientId;
    ifFrame.id = "serverIFrame_" + streamId;
    document.body.appendChild(ifFrame);
    ifFrame.style.width='1px';
    ifFrame.style.height='1px';
    ifFrame.style.display='block';
    ifFrame.style.visibility='hidden';
    ifFrame.style.position='absolute';        

    // automatically reconnecting every 30 seconds
    // (This is not like autoReconnect which reconnects after a server side stream close) 
    initReconnectTimeout(streamId);
};

/**
 * Disconnect comet stream
 */
dropServerConnection = function(streamId) {
	console.log("dropServerConnection", streamId);
	
    // Stop reconnection procedure in case client itself asks for a stop 
    autoReconnect[streamId] = false;
    
    sendControlMessage(streamId, "KILL");    
};


