// ============================================================================
//
// Copyright (c) 2010 - 2011, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ===================================================================


package ch.niceideas.common.utils;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.List;

import junit.framework.TestCase;

import org.junit.Test;

public class ReflectionUtilsTest extends TestCase {

    @Test
    public void testGetGetters() {

        List<Method> methodList = ReflectionUtils.getGetters(DummyBO.class, true);

        boolean getBidonStringFound = false;
        boolean getBidonLongFound = false;
        boolean getTransientStringFound = false;
        boolean getIdFound = false;

        for (Method method : methodList) {
            if (method.getName().equals("getBidonString")) {
                getBidonStringFound = true;
            }
            if (method.getName().equals("getBidonLong")) {
                getBidonLongFound = true;
            }
            if (method.getName().equals("getTransientString")) {
                getTransientStringFound = true;
            }
            if (method.getName().equals("getId")) {
                getIdFound = true;
            }
        }

        assertTrue(getBidonStringFound);
        assertTrue(getBidonLongFound);
        assertTrue(getTransientStringFound);
        assertTrue(getIdFound);
    }

    @Test
    public void testGetGettersNoTransient() {

        List<Method> methodList = ReflectionUtils.getGetters(DummyBO.class, false);

        boolean getBidonStringFound = false;
        boolean getBidonLongFound = false;
        boolean getTransientStringFound = false;
        boolean getIdFound = false;

        for (Method method : methodList) {
            if (method.getName().equals("getBidonString")) {
                getBidonStringFound = true;
            }
            if (method.getName().equals("getBidonLong")) {
                getBidonLongFound = true;
            }
            if (method.getName().equals("getTransientString")) {
                getTransientStringFound = true;
            }
            if (method.getName().equals("getId")) {
                getIdFound = true;
            }
        }

        assertTrue(getBidonStringFound);
        assertTrue(getBidonLongFound);
        assertTrue(!getTransientStringFound);
        assertTrue(getIdFound);
    }

    @Test
    public void testGetSetters() {

        List<Method> methodList = ReflectionUtils.getSetters(DummyBO.class, true);

        boolean setBidonStringFound = false;
        boolean setBidonLongFound = false;
        boolean setTransientStringFound = false;

        for (Method method : methodList) {
            if (method.getName().equals("setBidonString")) {
                setBidonStringFound = true;
            }
            if (method.getName().equals("setBidonLong")) {
                setBidonLongFound = true;
            }
            if (method.getName().equals("setTransientString")) {
                setTransientStringFound = true;
            }
        }

        assertTrue(setBidonStringFound);
        assertTrue(setBidonLongFound);
        assertTrue(setTransientStringFound);
    }

    @Test
    public void testGetSettersNoTransient() {

        List<Method> methodList = ReflectionUtils.getSetters(DummyBO.class, false);

        boolean setBidonStringFound = false;
        boolean setBidonLongFound = false;
        boolean setTransientStringFound = false;

        for (Method method : methodList) {
            if (method.getName().equals("setBidonString")) {
                setBidonStringFound = true;
            }
            if (method.getName().equals("setBidonLong")) {
                setBidonLongFound = true;
            }
            if (method.getName().equals("setTransientString")) {
                setTransientStringFound = true;
            }
        }

        assertTrue(setBidonStringFound);
        assertTrue(setBidonLongFound);
        assertTrue(!setTransientStringFound);
    }

    @Test
    public void testGetMethod() throws Exception {

        Method method = ReflectionUtils.getMethod(DummyBO.class, "setBidonLong", new Class<?>[] { Long.class });
        assertNotNull(method);
        assertEquals("setBidonLong", method.getName());

        try {
            method = ReflectionUtils.getMethod(DummyBO.class, "setTrucMachin", new Class<?>[] { Long.class });
            fail("NoSuchMethodError expected");
        } catch (NoSuchMethodException e) {
            assertNotNull(e);
        }
    }

    @Test
    public void testGetField() throws Exception {

        Field field = ReflectionUtils.getField(DummyBO.class, "bidonLong");
        assertNotNull(field);
        assertEquals("bidonLong", field.getName());

        try {
            field = ReflectionUtils.getField(DummyBO.class, "trucMachin");
            fail("NoSuchFieldException expected");
        } catch (NoSuchFieldException e) {
            assertNotNull(e);
        }
    }

    @Test
    public void testGetGetter() throws Exception {

        Method method = ReflectionUtils.getGetter(DummyBO.class, "bidonLong");
        assertNotNull(method);
        assertEquals("getBidonLong", method.getName());

        try {
            method = ReflectionUtils.getGetter(DummyBO.class, "trucMachin");
            fail("NoSuchMethodError expected");
        } catch (NoSuchMethodException e) {
            assertNotNull(e);
        }
    }

    @Test
    public void testGetSetter() throws Exception {

        Method method = ReflectionUtils.getSetter(DummyBO.class, "bidonLong", Long.class);
        assertNotNull(method);
        assertEquals("setBidonLong", method.getName());

        try {
            method = ReflectionUtils.getSetter(DummyBO.class, "trucMachin", Long.class);
            fail("NoSuchMethodError expected");
        } catch (NoSuchMethodException e) {
            assertNotNull(e);
        }
    }

    @Test
    public void testGetValueFromPath() {

        DummyBO bidon = new DummyBO();
        bidon.setBidonString("essai");

        // 1. test composed path
        byte[] originalBytes = bidon.getBidonString().getBytes();
        byte[] exctractedBytes = (byte[]) ReflectionUtils.getValueFromPath(bidon, "bidonString.bytes");

        assertEquals(originalBytes.length, exctractedBytes.length);

        for (int i = 0; i < originalBytes.length; i++) {
            assertEquals(originalBytes[i], exctractedBytes[i]);
        }

        // 2. test simple path
        String originalString = bidon.getBidonString();
        String exctractedString = (String) ReflectionUtils.getValueFromPath(bidon, "bidonString");

        assertEquals(originalString, exctractedString);

        // 2. TestCount identity path
        DummyBO second = (DummyBO) ReflectionUtils.getValueFromPath(bidon, ".");

        assertEquals(bidon, second);
    }

    @Test
    public void testSetFailsafeFieldValue() throws Exception {

        Field staticFinalField = ReflectionUtils.getField(StaticFinalFieldHolder.class, "stringArray");
        assertNotNull(staticFinalField);

        ReflectionUtils.setFailsafeFieldValue(staticFinalField, null, new String[] { "jerome", "badtrash" });

        String[] stringArray = (String[]) staticFinalField.get(StaticFinalFieldHolder.class);
        assertNotNull(stringArray);

        assertEquals(2, stringArray.length);
    }

    public static class StaticFinalFieldHolder {

        @SuppressWarnings("unused")
        private static String[] stringArray;
    }

    public static class DummyBO {

        private DummyID id = new DummyID();

        private String bidonString;
        private String bidonLong;
        private transient String transientString;

        public DummyID getId() {
            return id;
        }

        public void setId(DummyID id) {
            this.id = id;
        }

        public String getTransientString() {
            return transientString;
        }

        public void setTransientString(String transientString) {
            this.transientString = transientString;
        }

        public String getBidonLong() {
            return bidonLong;
        }

        public void setBidonLong(String bidonLong) {
            this.bidonLong = bidonLong;
        }

        public String getBidonString() {
            return bidonString;
        }

        public void setBidonString(String bidonString) {
            this.bidonString = bidonString;
        }
    }

    public static class DummyID {

        private String myString;
        private Long myLong;

        public String getMyString() {
            return myString;
        }

        public void setMyString(String myString) {
            this.myString = myString;
        }

        public Long getMyLong() {
            return myLong;
        }

        public void setMyLong(Long myLong) {
            this.myLong = myLong;
        }
    }

}
